@@ -1,7 +1,7 @@
 """
 PySCeS - Python Simulator for Cellular Systems (http://pysces.sourceforge.net)
 
-Copyright (C) 2004-2013 B.G. Olivier, J.M. Rohwer, J.-H.S Hofmeyr all rights reserved,
+Copyright (C) 2004-2020 B.G. Olivier, J.M. Rohwer, J.-H.S Hofmeyr all rights reserved,
 
 Brett G. Olivier (bgoli@users.sourceforge.net)
 Triple-J Group for Molecular Cell Physiology
@@ -14,6 +14,9 @@ this distribution for specifics.
 NO WARRANTY IS EXPRESSED OR IMPLIED.  USE AT YOUR OWN RISK.
 Brett G. Olivier
 """
+from __future__ import division, print_function
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
 from pysces.version import __version__
 __doc__ = '''PySCeS unittest module'''
@@ -48,8 +51,8 @@ def CopyTestModels(dirIn=os.path.join(pysces.install_dir,'pscmodels'),dirOut=pys
 
     if os.path.exists(dirIn):
         if os.path.exists(dirOut):
-            print 'src : ' + dirIn
-            print 'dest: ' + dirOut
+            print('src : ' + dirIn)
+            print('dest: ' + dirOut)
             flist = os.listdir(dirIn)
             if overwrite == 0:
                 flist2 = os.listdir(dirOut)
@@ -65,16 +68,16 @@ def CopyTestModels(dirIn=os.path.join(pysces.install_dir,'pscmodels'),dirOut=pys
                                 #print File +  (maxlen-len(File))*'.' + ' skipped'
                             except:
                                 shutil.copy(os.path.join(dirIn,File),os.path.join(dirOut,File))
-                                print File +  (maxlen-len(File))*'.' + ' ok'
+                                print(File +  (maxlen-len(File))*'.' + ' ok')
                         else:
                             shutil.copy(os.path.join(dirIn,File),os.path.join(dirOut,File))
-                            print File +  (maxlen-len(File))*'.' + ' ok'
+                            print(File +  (maxlen-len(File))*'.' + ' ok')
             else:
-                print 'Empty directory?'
+                print('Empty directory?')
         else:
-            print dirOut + ' does not exist'
+            print(dirOut + ' does not exist')
     else:
-        print dirIn + ' does not exist'
+        print(dirIn + ' does not exist')
 
 
 class PyscesTest:
@@ -84,9 +87,9 @@ class PyscesTest:
 
     def __init__(self,lvl=2,std2file=0):
         #copy models from server to local model store
-        print '\nCopying pysces_test models if necessary ...'
+        print('\nCopying pysces_test models if necessary ...')
         CopyTestModels()
-        print 'done.'
+        print('done.')
         self.basic_runner = unittest.TextTestRunner()
         self.BasicTest = unittest.makeSuite(PyscesBasicTest,'test')
         self.ExtendedTest = unittest.makeSuite(PyscesExtendedTest,'test')
@@ -97,7 +100,7 @@ class PyscesTest:
         class NullWriter:
             def __init__(self,dres=0):
                 if dres:
-                    self.Fout = file(os.path.join(os.getcwd(),'pysces_test_results.txt'),'w')
+                    self.Fout = open(os.path.join(os.getcwd(),'pysces_test_results.txt'),'w')
                     self.fof = 1
                 else:
                     self.fof = 0
@@ -113,7 +116,7 @@ class PyscesTest:
         self.__dfi__ = NullWriter(std2file)
         tmpSTD = sys.stdout
         for x in range(lvl):
-            print '\nLevel ' + str(x+1) + ' tests'
+            print('\nLevel ' + str(x+1) + ' tests')
             sys.stdout = self.__dfi__
             getattr(self, 'lvl_'+str(x+1))()
             sys.stdout = tmpSTD
@@ -468,18 +471,18 @@ class PyscesExternalTest(unittest.TestCase):
 
     def test_PITCON1(self):
         import scipy
-        print '''
+        print('''
         C  PCPRB1.FOR  The Freudenstein-Roth function.
         C  Limit points in the first variable occur at:
         C    (14.28309, -1.741377,  0.2585779)
         C    (61.66936,  1.983801, -0.6638797)
-        '''
+        ''')
         def fx(X):
             FX[0] = X[0] - ((X[1] - 5.0)*X[1] + 2.0) * X[1] - 13.0 + 34.0 * (X[2]-1.0)
             FX[1] = X[0] + ((X[1] + 1.0)*X[1] - 14.0) * X[1] - 29.0 + 10.0 *(X[2] - 1.0)
             return FX
         def fjac(s):
-             return
+            return
 
         parm = 3; iwork = scipy.zeros((30 + parm),'i');    rwork = scipy.zeros((30 + (6*parm)*parm),'d')
         ipar = scipy.zeros((parm),'i');    fpar = scipy.zeros((parm),'d');    xr = scipy.zeros((parm),'d') #output array
@@ -499,8 +502,8 @@ class PyscesExternalTest(unittest.TestCase):
         for run in range(iterations):
             ierror,iwork,rwork,xrout = pysces.pitcon.pitcon1(fjac,fpar,fx,ipar,iwork,rwork,xr)
             if iwork[0] == 4:
-                print '\nLimit point in run = ' + `run + 1`
-                print xrout
+                print('\nLimit point in run = ' + repr(run + 1))
+                print(xrout)
                 limits.append(xrout)
             output.append(xrout)
 
@@ -516,60 +519,60 @@ class PyscesExternalTest(unittest.TestCase):
         assert_array_almost_equal(limits[1],limit1,4)
         assert_array_almost_equal(output6,out6,4)
 
-    ##  def test_PITCON2(self): # unreliable
-        ##  mod = pysces.model('pysces_test_pitcon.psc', self.model_dir)
-        ##  mod.doLoad()
-        ##  mod.V2 = mod.V3 = 100.0
-        ##  mod.A05 = 10.0
-        ##  mod.pitcon_iter = 5
-        ##  mod.pitcon_par_space = logspace(0,1,10)
-        ##  res = mod.PITCON('P')
-        ##  # pysces.plt.plot2D(res, 0, style='.'); pysces.plt.logxy()
+    #def test_PITCON2(self): # unreliable
+        #mod = pysces.model('pysces_test_pitcon.psc', self.model_dir)
+        #mod.doLoad()
+        #mod.V2 = mod.V3 = 100.0
+        #mod.A05 = 10.0
+        #mod.pitcon_iter = 5
+        #mod.pitcon_par_space = logspace(0,1,10)
+        #res = mod.PITCON('P')
+        ## pysces.plt.plot2D(res, 0, style='.'); pysces.plt.logxy()
 
-        ##  os.sys.stderr.write('\n\"Iteration terminates after NITMAX and Newton method fails to converge\" warnings can be ignored\n')
-        ##  benchmark = array([
-        ##  [8.39949877e-01,1.61211501e+01,4.84068424e+00,7.12011640e+01,7.12011640e+01,7.12011640e+01],
-        ##  [5.40984421e-01,1.57720807e+01,4.31351073e+00,7.27545617e+01,7.27545617e+01,7.27545617e+01],
-        ##  [-5.14362451e-02,1.45210385e+01,3.06361620e+00,7.64860963e+01,7.64860963e+01,7.64860963e+01],
-        ##  [-4.66310864e-01,1.31962735e+01,1.95980785e+00,8.04668684e+01,8.04668684e+01,8.04668683e+01],
-        ##  [1.13190191e+00,1.61011691e+01,5.26326909e+00,6.96411063e+01,6.96411063e+01,6.96411063e+01],
-        ##  [8.30873009e-01,1.61149804e+01,4.82588194e+00,7.12478479e+01,7.12478479e+01,7.12478479e+01],
-        ##  [4.39606946e-01,1.56001789e+01,4.11842961e+00,7.33077027e+01,7.33077027e+01,7.33077027e+01],
-        ##  [-3.07339668e-01,1.37808508e+01,2.41799659e+00,7.87206889e+01,7.87206889e+01,7.87206889e+01],
-        ##  [1.64608357e+00,1.37635551e+01,5.46388538e+00,6.53427535e+01,6.53427535e+01,6.53427535e+01],
-        ##  [1.57039736e+00,1.45248830e+01,5.53382469e+00,6.63454792e+01,6.63454792e+01,6.63454792e+01],
-        ##  [1.36033858e+00,1.56423070e+01,5.48938893e+00,6.81979734e+01,6.81979734e+01,6.81979734e+01],
-        ##  [3.90033764e-01,1.55086998e+01,4.02030887e+00,7.35869382e+01,7.35869382e+01,7.35869382e+01],
-        ##  [1.90300355e+00,4.61218050e-01,8.11483528e-01,1.67232557e+01,1.67232557e+01,1.67232557e+01],
-        ##  [1.65604156e+00,1.14527548e+00,1.32711044e+00,2.91604808e+01,2.91604810e+01,2.91604810e+01],
-        ##  [1.60792245e+00,3.36846985e+00,2.47903648e+00,4.55723011e+01,4.55723011e+01,4.55723011e+01],
-        ##  [1.76260920e+00,1.02529273e+01,4.77285351e+00,6.09994737e+01,6.09994737e+01,6.09994737e+01],
-        ##  [2.48410121e+00,1.76577346e-01,5.36816396e-01,7.17264824e+00,7.17264824e+00,7.17264824e+00],
-        ##  [2.04216203e+00,3.36094901e-01,6.94802141e-01,1.31279273e+01,1.31279273e+01,1.31279273e+01],
-        ##  [1.68318189e+00,9.87364532e-01,1.22120154e+00,2.69666997e+01,2.69666997e+01,2.69666997e+01],
-        ##  [1.60195518e+00,3.09604637e+00,2.35831199e+00,4.43144773e+01,4.43144773e+01,4.43144773e+01],
-        ##  [3.29451248e+00,1.10989590e-01,5.07996700e-01,3.71775348e+00,3.71775351e+00,3.71775348e+00],
-        ##  [2.53835638e+00,1.67389490e-01,5.28506741e-01,6.75388116e+00,6.75388117e+00,6.75388117e+00],
-        ##  [2.13066473e+00,2.84747678e-01,6.44229065e-01,1.14218470e+01,1.14218470e+01,1.14218470e+01],
-        ##  [1.85930898e+00,5.18944892e-01,8.62231652e-01,1.81726016e+01,1.81726016e+01,1.81726016e+01],
-        ##  [4.93995542e+00,1.06865480e-01,6.50716299e-01,2.37791780e+00,2.37791781e+00,2.37791780e+00],
-        ##  [5.51998987e+00,1.12930767e-01,7.15671948e-01,2.26203167e+00,2.26203167e+00,2.26203167e+00],
-        ##  [7.20197168e+00,1.35245848e-01,9.14818799e-01,2.13476038e+00,2.13476038e+00,2.13476038e+00],
-        ##  [1.21464994e+01,2.10315787e-01,1.52485159e+00,2.11433598e+00,2.11433598e+00,2.11433598e+00],
-        ##  [6.29277796e+00,1.22633783e-01,8.05857247e-01,2.18036060e+00,2.18036060e+00,2.18036060e+00],
-        ##  [6.71463987e+00,1.28373921e-01,8.56137906e-01,2.15469266e+00,2.15469266e+00,2.15469266e+00],
-        ##  [7.96638113e+00,1.46372121e-01,1.00775931e+00,2.11668562e+00,2.11668562e+00,2.11668562e+00],
-        ##  [1.16768618e+01,2.02994936e-01,1.46632082e+00,2.11150946e+00,2.11150946e+00,2.11150946e+00],
-        ##  [8.04040339e+00,1.47466151e-01,1.01680363e+00,2.11553054e+00,2.11553054e+00,2.11553055e+00],
-        ##  [8.46158253e+00,1.53734006e-01,1.06838323e+00,2.11040547e+00,2.11040547e+00,2.11040547e+00],
-        ##  [9.21463978e+00,1.65086727e-01,1.16102019e+00,2.10586653e+00,2.10586653e+00,2.10586653e+00],
-        ##  [1.07956247e+01,1.89324600e-01,1.35672050e+00,2.10728985e+00,2.10728985e+00,2.10728985e+00],
-        ##  [1.02976931e+01,1.81645169e-01,1.29494069e+00,2.10576586e+00,2.10576586e+00,2.10576586e+00],
-        ##  [1.07186771e+01,1.88135560e-01,1.34716589e+00,2.10700665e+00,2.10700665e+00,2.10700665e+00],
-        ##  [1.13140312e+01,1.97355266e-01,1.42115741e+00,2.10957637e+00,2.10957637e+00,2.10957637e+00],
-        ##  [1.21560089e+01,2.10464256e-01,1.52603757e+00,2.11439651e+00,2.11439651e+00,2.11439651e+00]])
-        ##  assert_array_almost_equal(benchmark[:5,:],res[:5,:],3)
-        ##  assert_array_almost_equal(benchmark[-5:,:],res[-5:,:],1)
+        #os.sys.stderr.write('\n\"Iteration terminates after NITMAX and Newton method fails to converge\" warnings can be ignored\n')
+        #benchmark = array([
+            #[8.39949877e-01,1.61211501e+01,4.84068424e+00,7.12011640e+01,7.12011640e+01,7.12011640e+01],
+            #[5.40984421e-01,1.57720807e+01,4.31351073e+00,7.27545617e+01,7.27545617e+01,7.27545617e+01],
+            #[-5.14362451e-02,1.45210385e+01,3.06361620e+00,7.64860963e+01,7.64860963e+01,7.64860963e+01],
+            #[-4.66310864e-01,1.31962735e+01,1.95980785e+00,8.04668684e+01,8.04668684e+01,8.04668683e+01],
+            #[1.13190191e+00,1.61011691e+01,5.26326909e+00,6.96411063e+01,6.96411063e+01,6.96411063e+01],
+            #[8.30873009e-01,1.61149804e+01,4.82588194e+00,7.12478479e+01,7.12478479e+01,7.12478479e+01],
+            #[4.39606946e-01,1.56001789e+01,4.11842961e+00,7.33077027e+01,7.33077027e+01,7.33077027e+01],
+            #[-3.07339668e-01,1.37808508e+01,2.41799659e+00,7.87206889e+01,7.87206889e+01,7.87206889e+01],
+            #[1.64608357e+00,1.37635551e+01,5.46388538e+00,6.53427535e+01,6.53427535e+01,6.53427535e+01],
+            #[1.57039736e+00,1.45248830e+01,5.53382469e+00,6.63454792e+01,6.63454792e+01,6.63454792e+01],
+            #[1.36033858e+00,1.56423070e+01,5.48938893e+00,6.81979734e+01,6.81979734e+01,6.81979734e+01],
+            #[3.90033764e-01,1.55086998e+01,4.02030887e+00,7.35869382e+01,7.35869382e+01,7.35869382e+01],
+            #[1.90300355e+00,4.61218050e-01,8.11483528e-01,1.67232557e+01,1.67232557e+01,1.67232557e+01],
+            #[1.65604156e+00,1.14527548e+00,1.32711044e+00,2.91604808e+01,2.91604810e+01,2.91604810e+01],
+            #[1.60792245e+00,3.36846985e+00,2.47903648e+00,4.55723011e+01,4.55723011e+01,4.55723011e+01],
+            #[1.76260920e+00,1.02529273e+01,4.77285351e+00,6.09994737e+01,6.09994737e+01,6.09994737e+01],
+            #[2.48410121e+00,1.76577346e-01,5.36816396e-01,7.17264824e+00,7.17264824e+00,7.17264824e+00],
+            #[2.04216203e+00,3.36094901e-01,6.94802141e-01,1.31279273e+01,1.31279273e+01,1.31279273e+01],
+            #[1.68318189e+00,9.87364532e-01,1.22120154e+00,2.69666997e+01,2.69666997e+01,2.69666997e+01],
+            #[1.60195518e+00,3.09604637e+00,2.35831199e+00,4.43144773e+01,4.43144773e+01,4.43144773e+01],
+            #[3.29451248e+00,1.10989590e-01,5.07996700e-01,3.71775348e+00,3.71775351e+00,3.71775348e+00],
+            #[2.53835638e+00,1.67389490e-01,5.28506741e-01,6.75388116e+00,6.75388117e+00,6.75388117e+00],
+            #[2.13066473e+00,2.84747678e-01,6.44229065e-01,1.14218470e+01,1.14218470e+01,1.14218470e+01],
+            #[1.85930898e+00,5.18944892e-01,8.62231652e-01,1.81726016e+01,1.81726016e+01,1.81726016e+01],
+            #[4.93995542e+00,1.06865480e-01,6.50716299e-01,2.37791780e+00,2.37791781e+00,2.37791780e+00],
+            #[5.51998987e+00,1.12930767e-01,7.15671948e-01,2.26203167e+00,2.26203167e+00,2.26203167e+00],
+            #[7.20197168e+00,1.35245848e-01,9.14818799e-01,2.13476038e+00,2.13476038e+00,2.13476038e+00],
+            #[1.21464994e+01,2.10315787e-01,1.52485159e+00,2.11433598e+00,2.11433598e+00,2.11433598e+00],
+            #[6.29277796e+00,1.22633783e-01,8.05857247e-01,2.18036060e+00,2.18036060e+00,2.18036060e+00],
+            #[6.71463987e+00,1.28373921e-01,8.56137906e-01,2.15469266e+00,2.15469266e+00,2.15469266e+00],
+            #[7.96638113e+00,1.46372121e-01,1.00775931e+00,2.11668562e+00,2.11668562e+00,2.11668562e+00],
+            #[1.16768618e+01,2.02994936e-01,1.46632082e+00,2.11150946e+00,2.11150946e+00,2.11150946e+00],
+            #[8.04040339e+00,1.47466151e-01,1.01680363e+00,2.11553054e+00,2.11553054e+00,2.11553055e+00],
+            #[8.46158253e+00,1.53734006e-01,1.06838323e+00,2.11040547e+00,2.11040547e+00,2.11040547e+00],
+            #[9.21463978e+00,1.65086727e-01,1.16102019e+00,2.10586653e+00,2.10586653e+00,2.10586653e+00],
+            #[1.07956247e+01,1.89324600e-01,1.35672050e+00,2.10728985e+00,2.10728985e+00,2.10728985e+00],
+            #[1.02976931e+01,1.81645169e-01,1.29494069e+00,2.10576586e+00,2.10576586e+00,2.10576586e+00],
+            #[1.07186771e+01,1.88135560e-01,1.34716589e+00,2.10700665e+00,2.10700665e+00,2.10700665e+00],
+            #[1.13140312e+01,1.97355266e-01,1.42115741e+00,2.10957637e+00,2.10957637e+00,2.10957637e+00],
+            #[1.21560089e+01,2.10464256e-01,1.52603757e+00,2.11439651e+00,2.11439651e+00,2.11439651e+00]])
+        #assert_array_almost_equal(benchmark[:5,:],res[:5,:],3)
+        #assert_array_almost_equal(benchmark[-5:,:],res[-5:,:],1)
 
 
 
