@@ -3,7 +3,7 @@
 """
 PySCeS - Python Simulator for Cellular Systems (http://pysces.sourceforge.net)
 
-Copyright (C) 2004-2013 B.G. Olivier, J.M. Rohwer, J.-H.S Hofmeyr all rights reserved,
+Copyright (C) 2004-2020 B.G. Olivier, J.M. Rohwer, J.-H.S Hofmeyr all rights reserved,
 
 Brett G. Olivier (bgoli@users.sourceforge.net)
 Triple-J Group for Molecular Cell Physiology
@@ -24,4 +24,4 @@ Adapted from the original NumPy src (numpy.scipy.org).
 """
 FRYING_EGGS = True
 from setuptools import setup
-execfile('setup.py')
+exec(compile(open('setup.py').read(), 'setup.py', 'exec'))
